@@ -25,10 +25,10 @@ operations on the Accessible object, such as invoking default actions or
 moving the text caret.
 """
 
-__id__        = "$Id: braille.py 2520 2007-07-06 19:51:57Z joanied $"
-__version__   = "$Revision: 2520 $"
-__date__      = "$Date: 2007-07-06 15:51:57 -0400 (Fri, 06 Jul 2007) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: braille.py 2538 2007-07-12 19:40:46Z joanied $"
+__version__   = "$Revision: 2538 $"
+__date__      = "$Date: 2007-07-12 15:40:46 -0400 (Thu, 12 Jul 2007) $"
+__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import logging
@@ -503,7 +503,7 @@ class Text(Region):
                 for i in range(0, nSelections):
                     [start, end] = text.getSelection(i)
                     maskStart = max(self.lineOffset, start) - self.lineOffset
-                    maskEnd = end - self.lineOffset
+                    maskEnd = min(end - self.lineOffset, stringLength)
                     # Combine the selection indicator with the attribute
                     # indicator.
                     #
@@ -831,12 +831,11 @@ def refresh(panToCursor=True, targetCursorCell=0):
         cursorCell += 1 # Normalize to 1-based offset
 
     debug.println(debug.LEVEL_INFO, "BRAILLE LINE:  '%s'" % string)
-    log.info("line:'%s'" % string)
-
     debug.println(debug.LEVEL_INFO, "     VISIBLE:  '%s', cursor=%d" \
                   % (string[startPos:endPos], cursorCell))
-    log.info("visible:'%s'" % string[startPos:endPos])
-    log.info("cursor:%d" % cursorCell)
+
+    log.info("refresh line='%s', visible='%s', cursor=%d" \
+             % (string, string[startPos:endPos], cursorCell))
 
     string = string.decode("UTF-8")
     substring = string[startPos:endPos].encode("UTF-8")
