@@ -23,9 +23,9 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 2539 2007-07-13 18:32:02Z richb $"
-__version__   = "$Revision: 2539 $"
-__date__      = "$Date: 2007-07-13 14:32:02 -0400 (Fri, 13 Jul 2007) $"
+__id__        = "$Id: braillegenerator.py 3011 2007-10-10 20:27:20Z joanied $"
+__version__   = "$Revision: 3011 $"
+__date__      = "$Date: 2007-10-10 16:27:20 -0400 (Wed, 10 Oct 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -107,6 +107,8 @@ class BrailleGenerator:
              self._getBrailleRegionsForPageTab
         self.brailleGenerators[rolenames.ROLE_PAGE_TAB_LIST]       = \
              self._getBrailleRegionsForPageTabList
+        self.brailleGenerators[rolenames.ROLE_PANEL]               = \
+             self._getBrailleRegionsForPanel
         self.brailleGenerators[rolenames.ROLE_PARAGRAPH]           = \
              self._getBrailleRegionsForText
         self.brailleGenerators[rolenames.ROLE_PASSWORD_TEXT]       = \
@@ -839,6 +841,40 @@ class BrailleGenerator:
 
         return self._getDefaultBrailleRegions(obj)
 
+    def _getBrailleRegionsForPanel(self, obj):
+        """Gets text to be displayed for a panel.
+
+        Arguments:
+        - obj: an Accessible
+
+        Returns a list where the first element is a list of Regions to
+        display and the second element is the Region which should get
+        focus.
+        """
+
+        self._debugGenerator("_getBrailleRegionsForPanel", obj)
+
+        regions = []
+
+        text = ""
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+
+        # If there was no label for the panel, but it has a name, we'll
+        # use the name.
+        #
+        if len(text) == 0:
+            text = self._script.appendString(
+                text, self._script.getDisplayedText(obj))
+
+        text = self._script.appendString(text, self._getTextForRole(obj))
+
+        regions = []
+        componentRegion = braille.Component(obj, text)
+        regions.append(componentRegion)
+
+        return [regions, componentRegion]
+
     def _getBrailleRegionsForProgressBar(self, obj):
         """Get the braille for a progress bar.  If the object already
         had focus, just the new value is displayed.
@@ -1135,7 +1171,7 @@ class BrailleGenerator:
                     obj.role = rolenames.ROLE_CHECK_BOX
                     regions = self._getBrailleRegionsForCheckBox(obj)
 
-                    # If this table cell doesn't have any label associated 
+                    # If this table cell doesn't have any label associated
                     # with it then also braille the table column header.
                     # See Orca bug #455230 for more details.
                     #
@@ -1160,6 +1196,23 @@ class BrailleGenerator:
                 self._script.getRealActiveDescendant(obj))
             regions[0].extend(cellRegions)
 
+        # Check to see if this table cell contains an icon (image).
+        # If yes:
+        #   1/ Try to get a description for it and speak that.
+        #   2/ Treat the object of role type ROLE_IMAGE and speak
+        #      the role name.
+        # See bug #465989 for more details.
+        #
+        displayedText = self._script.getDisplayedText( \
+                          self._script.getRealActiveDescendant(obj))
+        if (not displayedText or len(displayedText) == 0) and obj.image:
+            if obj.image.imageDescription:
+                regions[0].append(obj.image.imageDescription)
+            obj.role = rolenames.ROLE_IMAGE
+            [cellRegions, focusRegion] = self._getBrailleRegionsForImage(obj)
+            regions[0].extend(cellRegions)
+            obj.role = rolenames.ROLE_TABLE_CELL
+
         # [[[TODO: WDW - HACK attempt to determine if this is a node;
         # if so, describe its state.]]]
         #
@@ -1177,14 +1230,6 @@ class BrailleGenerator:
                 #
                 regions[0].append(braille.Region(" " + _("collapsed")))
 
-        level = self._script.getNodeLevel(obj)
-        if level >= 0:
-            # Translators: this represents the depth of a node in a tree
-            # view (i.e., how many ancestors a node has).
-            #
-            regions[0].append(braille.Region(" " + _("TREE LEVEL %d") \
-                                             % (level + 1)))
-
         return regions
 
     def _getBrailleRegionsForTableCellRow(self, obj):
@@ -1245,12 +1290,20 @@ class BrailleGenerator:
                         if i == column:
                             focusRowRegion = cellRegions[0]
                 regions = [rowRegions, focusRowRegion]
-                settings.brailleVerbosityLevel = savedBrailleVerbosityLevel
             else:
                 regions = self._getBrailleRegionsForTableCell(obj)
+            settings.brailleVerbosityLevel = savedBrailleVerbosityLevel
         else:
             regions = self._getBrailleRegionsForTableCell(obj)
 
+        level = self._script.getNodeLevel(obj)
+        if level >= 0:
+            # Translators: this represents the depth of a node in a tree
+            # view (i.e., how many ancestors a node has).
+            #
+            regions[0].append(braille.Region(" " + _("TREE LEVEL %d") \
+                                             % (level + 1)))
+
         return regions
 
     def _getBrailleRegionsForTableColumnHeader(self, obj):
@@ -1552,7 +1605,9 @@ class BrailleGenerator:
         parent = obj.parent
         if parent and parent.table:
             row = parent.table.getRowAtIndex(obj.index)
-            if row >= 0:
+            if (row >= 0) \
+                and (not obj.role in [rolenames.ROLE_ROW_HEADER,
+                                      rolenames.ROLE_TABLE_ROW_HEADER]):
                 desc = parent.table.getRowDescription(row)
             else:
                 desc = None
@@ -1571,7 +1626,9 @@ class BrailleGenerator:
                 regions.append(braille.Region(text))
 
             col = parent.table.getColumnAtIndex(obj.index)
-            if col >= 0:
+            if (col >= 0) \
+                and (not obj.role in [rolenames.ROLE_COLUMN_HEADER,
+                                      rolenames.ROLE_TABLE_COLUMN_HEADER]):
                 desc = parent.table.getColumnDescription(col)
             else:
                 desc = None
