@@ -19,9 +19,9 @@
 
 """Provides a SpeechServer factory for gnome-speech drivers."""
 
-__id__        = "$Id: gnomespeechfactory.py 2518 2007-07-03 19:12:15Z richb $"
-__version__   = "$Revision: 2518 $"
-__date__      = "$Date: 2007-07-03 15:12:15 -0400 (Tue, 03 Jul 2007) $"
+__id__        = "$Id: gnomespeechfactory.py 2558 2007-07-27 18:48:34Z richb $"
+__version__   = "$Revision: 2558 $"
+__date__      = "$Date: 2007-07-27 14:48:34 -0400 (Fri, 27 Jul 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -96,6 +96,8 @@ class _Speaker(GNOME__POA.Speech.SpeechCallback):
             callback.notify(type, id, offset)
 
     def say(self, text):
+        if isinstance(text, unicode):
+            text = text.encode("UTF-8")
         return self.gnome_speaker.say(text)
 
     def stop(self):
@@ -504,7 +506,7 @@ class SpeechServer(speechserver.SpeechServer):
                                           "SPEECH OUTPUT: '" \
                                           + utterance \
                                           + "'")
-                            log.info("'%s'" % utterance)
+                            log.info("sayAll utterance='%s'" % utterance)
                             if utterance and len(utterance)\
                                and not utterance.isspace():
                                 self.__sayAll.idForCurrentContext = \
@@ -705,6 +707,7 @@ class SpeechServer(speechserver.SpeechServer):
         style = settings.verbalizePunctuationStyle
         newText = ''
         for i in range(0, len(oldText)):
+            self.textCharIndices.append(len(newText))
             try:
                 level, action = \
                     punctuation_settings.getPunctuationInfo(oldText[i])
@@ -739,7 +742,6 @@ class SpeechServer(speechserver.SpeechServer):
                     nextCharMatches = (oldText[i + 1] in string.digits or \
                                        oldText[i + 1] in currencySymbols)
 
-                self.textCharIndices.append(len(newText))
                 if oldText[i] == "-" and \
                    style != settings.PUNCTUATION_STYLE_NONE and \
                    prevCharMatches and nextCharMatches:
@@ -765,7 +767,6 @@ class SpeechServer(speechserver.SpeechServer):
                 else:
                     newText += oldText[i].encode("UTF-8")
             except:
-                self.textCharIndices.append(len(newText))
                 if (len(oldText) == 1):
                     newText += chnames.getCharacterName(oldText[i])
                 else:
@@ -809,7 +810,7 @@ class SpeechServer(speechserver.SpeechServer):
             return -1
 
         text = self.__addVerbalizedPunctuation(text)
-        if orca_state.activeScript:
+        if orca_state.activeScript and orca_state.usePronunciationDictionary:
             text = orca_state.activeScript.adjustForPronunciation(text)
 
         try:
@@ -868,7 +869,7 @@ class SpeechServer(speechserver.SpeechServer):
             [context, acss] = utteranceIterator.next()
             debug.println(debug.LEVEL_INFO,
                           "SPEECH OUTPUT: '" + context.utterance + "'")
-            log.info("'%s'" % context.utterance)
+            log.info("sayAll utterance='%s'" % context.utterance)
             self.__sayAll = _SayAll(utteranceIterator,
                                     context,
                                     self.__speak(context.utterance, acss),
