@@ -20,9 +20,9 @@
 """Provides support for defining keybindings and matching them to input
 events."""
 
-__id__        = "$Id: keybindings.py 2254 2007-04-05 13:21:02Z wwalker $"
-__version__   = "$Revision: 2254 $"
-__date__      = "$Date: 2007-04-05 09:21:02 -0400 (Thu, 05 Apr 2007) $"
+__id__        = "$Id: keybindings.py 2351 2007-05-07 21:56:41Z wwalker $"
+__version__   = "$Revision: 2351 $"
+__date__      = "$Date: 2007-05-07 17:56:41 -0400 (Mon, 07 May 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -42,9 +42,54 @@ import settings
 
 from orca_i18n import _           # for gettext support
 
+_keysymsCache = {}
 _keycodeCache = {}
 
-def _getKeycode(keysym):
+def getAllKeysyms(keysym):
+    """Given a keysym, find all other keysyms associated with the key
+    that is mapped to the given keysym.  This allows us, for example,
+    to determine that the key bound to KP_Insert is also bound to KP_0."""
+
+    if not _keysymsCache.has_key(keysym):
+        # The keysym itself is always part of the list.
+        #
+        _keysymsCache[keysym] = [keysym]
+
+        # Find the numerical value of the keysym
+        #
+        keyval = gtk.gdk.keyval_from_name(keysym)
+
+        if keyval != 0:
+            # Find the keycodes for the keysym.  Since a keysym
+            # can be associated with more than one key, we'll shoot
+            # for the keysym that's in group 0, regardless of shift
+            # level (each entry is of the form [keycode, group,
+            # level]).
+            #
+            keymap = gtk.gdk.keymap_get_default()
+            entries = keymap.get_entries_for_keyval(keyval)
+            keycode = 0
+            if entries:
+                for entry in entries:
+                    if entry[1] == 0:  # group = 0
+                        keycode = entry[0]
+                        break
+
+            # Find the keysyms bound to the keycode.  These are what
+            # we are looking for.
+            #
+            if keycode != 0:
+                entries = keymap.get_entries_for_keycode(keycode)
+                if entries:
+                    for entry in entries:
+                        keyval = entry[0]
+                        name = gtk.gdk.keyval_name(keyval)
+                        if name and (name != keysym):
+                            _keysymsCache[keysym].append(name)
+
+    return _keysymsCache[keysym]
+
+def getKeycode(keysym):
     """Converts an XKeysym string (e.g., 'KP_Enter') to a keycode that
     should match the event.hw_code for key events.
 
@@ -182,7 +227,7 @@ class KeyBinding:
         # keybindings in the user's preferences file.
         #
         if not self.keycode:
-            self.keycode = _getKeycode(self.keysymstring)
+            self.keycode = getKeycode(self.keysymstring)
 
         if self.keycode == keycode:
             result = modifiers & self.modifier_mask
@@ -197,6 +242,16 @@ class KeyBindings:
     def __init__(self):
         self.keyBindings = []
 
+    def __str__(self):
+        result = "[\n"
+        for keyBinding in self.keyBindings:
+            result += "  [%x %x %s %s]\n" % (keyBinding.modifier_mask,
+                                             keyBinding.modifiers,
+                                             keyBinding.keysymstring,
+                                             keyBinding.handler._description)
+        result += "]"
+        return result
+    
     def add(self, keyBinding):
         """Adds the given KeyBinding instance to this set of keybindings.
         """
